from Products.CMFCore.utils import getToolByName
from Products.CMFFormController.FormAction import FormActionKey
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.public import listTypes

from Acquisition import aq_base

from StringIO import StringIO

from Products.CMFDynamicViewFTI.migrate import migrateFTIs

from Products.COREBlog2.config import *
from Products.COREBlog2.COREBlogTool import COREBlog2Tool

def install(self):
    """Install COREBlog2: Install content types, skin layer, install the
    stylesheet, set up global properties, enable the portal factory and
    set up form controller actions for the widget actions
    """

    out = StringIO()

    # Install tool
    id = COREBlog2Tool.id
    if not hasattr(aq_base(self), id):
        addTool = self.manage_addProduct['COREBlog2'].manage_addTool
        addTool(COREBlog2Tool.meta_type)
        print >>out, 'Installing COREBlog2 Tool'
        

    print >> out, "Installing COREBlog2"

    # Install types
    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    print >> out, "Installed types"

    # Install skin
    install_subskin(self, out, GLOBALS)
    print >> out, "Installed skin"

    # Migrate FTI, to make sure we get the necessary infrastructure for the
    # 'display' menu to work.
    migrated = migrateFTIs(self, product=PROJECTNAME)
    print >>out, "Switched to DynamicViewFTI: %s" % ', '.join(migrated)

    # Enable portal_factory
    factory = getToolByName(self, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    for add_type in ['COREBlog2','COREBlogEntry','COREBlogCategory']:
        if add_type not in types:
            types.append(add_type)
            factory.manage_setPortalFactoryTypes(listOfTypeIds = types)

    print >> out, "Added COREBlog2 to portal_factory"

    #propsTool = getToolByName(self, 'portal_properties')
    #siteProperties = getattr(propsTool, 'site_properties')
    #navtreeProperties = getattr(propsTool, 'navtree_properties')

    return out.getvalue()


## Return referenced media content presentation
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=obj
##title=Return referenced attachment content presentation
##

# !!!
# Handling for Content-type, File:swf,mp3,mpeg4 or something
#

# Metal tempate for media handler
tp = container['cbattachment_view']
macro_keys = tp.macros.keys()

# First,see object portal_type and return appropreate template
if obj.portal_type in macro_keys:
    # The macro for object is there !
    return tp.macros[obj.portal_type]

if obj.portal_type.lower() in macro_keys:
    # The macro for object is there !
    return tp.macros[obj.portal_type.lower()]

# No custom template, so return object itself...
return tp.macros['Othrers__']






## Controller Python Script "resetCookie"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Receive trackback
##

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import log

cbtool = getToolByName(context, 'coreblog2_tool')

REQUEST = context.REQUEST
form = REQUEST.form
RESPONSE = context.REQUEST.RESPONSE
entry = context

excerpt = ''
if form.has_key('excerpt'):
    excerpt = form['excerpt']

title = cbtool.convert_charcode(form['title'])
blog_name = cbtool.convert_charcode(form['blog_name'])
excerpt = cbtool.convert_charcode(excerpt)

#Try to add trackback
try:
    #Send notify mail if need
    if context.getSend_trackback_notification():
        try:
            to_addr   = context.getNotify_to()
            from_addr = context.getNotify_to()
            msgbody = context.translate('trackback_notify_body')
            elements = {}
            for k in ('blog_name','title','excerpt','url','excerpt'):
                if form.has_key(k):
                    elements[k] = REQUEST.form[k]
                else:
                    elements[k] = ''
            elements['post_ip'] = REQUEST.getClientAddr()
            elements['entry_url'] = context.absolute_url()
            msgbody = msgbody % (elements)
            msgsubject = context.translate('trackback_notify_title')
            mgsheader = """To: %s
From: %s
Mime-Version: 1.0
Content-Type: text/plain; Charset=utf-8

""" % (to_addr,from_addr)
            cbtool.send_mail(mgsheader+msgbody, to_addr, from_addr, msgsubject)
        except Exception,e:
            log( 'COREBlog2/tbping: '
                     'Some exception occured, %s' % e )

    entry.addTrackback2Entry(title=title,url=form['url'],\
                            blog_name=blog_name,excerpt=excerpt)
    
    return context.tbping_result(client=context,REQUEST=REQUEST,\
                                        error_code=0,message='Thanks :-)')
except:
    return context.tbping_result(client=context,REQUEST=REQUEST,\
                                    error_code=1,message='Error occured!')




## Python Script "getEntryContentFilter"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Return ContentFilter for entry listing
##
REQ = context.REQUEST

# Initialize filter dictionary
filter = {'portal_type':['COREBlogEntry'],
          'sort_on':'Date',
          'sort_order':'reverse'}

if REQ.form.has_key('SearchableText') and REQ.form['SearchableText']:
    filter['SearchableText'] = context.REQUEST.form['SearchableText']

if REQ.form.has_key('maxDate') and REQ.form.has_key('minDate') and \
    REQ.form['maxDate'] and REQ.form['minDate']:
    try:
        filter['Date'] = (DateTime(REQ.form['maxDate']),
                          DateTime(REQ.form['minDate']))
        filter['Date_usage'] = "range:max min"
    except:
        pass
elif REQ.form.has_key('maxDate') and REQ.form['maxDate']:
    filter['Date'] = (DateTime(REQ.form['maxDate']),DateTime('1999-01-01'))
    filter['Date_usage'] = "range:max min"
elif REQ.form.has_key('minDate') and REQ.form['minDate']:
    filter['Date'] = (DateTime('2999-12-31'),DateTime(REQ.form['minDate']))
    filter['Date_usage'] = "range:max min"

# Return filter
return filter


## Python Script "getContentFilter"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Return ContentFilter,except for entry,CategoryFolder,CommentFolder
##

# Initialize filter dictionary
filter = {}

# Types to hide in contents tab
remove_types = ['COREBlogEntry',\
                'COREBlogCommentFolder','COREBlogCategoryFolder']

# Get types to allow to add in COREBlogFolder
types = [ tp.id for tp in context.getAllowedTypes()]

# Some folder automatically added, so add 'Folder' to types list
types.append('Folder')

# Loop over types to hide
for tp in remove_types:
    if tp in types:
        types.remove(tp)

filter['portal_type'] = types

# Return filter
return filter


##############################################################################
#
# __init__.py
# init file for COREBlog2 test
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: __init__.py 90 2005-11-25 10:37:43Z ats $
#
##############################################################################

"""COREBlog2 tests
"""

__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'



##############################################################################
#
# test_coreblog2.py
# Testcase Class for COREBlog2 Folder
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: test_coreblog2.py 189 2006-04-04 11:32:56Z ats $
#
##############################################################################

"""COREBlog2 tests
"""

__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

from Testing import ZopeTestCase
from DateTime import DateTime

from Products.COREBlog2.config import comment_folder_id,catetory_folder_id,\
                stuff_folder_id,images_folder_id


from coreblog2testcase import COREBlog2TestCase
klasses = []

class COREBlog2FolderTest(COREBlog2TestCase): 

    def test_000folders(self):
        #Test for standard folders existance
        ZopeTestCase._print('Test for standard folders ... \n')

        self.failUnless(comment_folder_id in self.blog.objectIds())
        self.failUnless(catetory_folder_id in self.blog.objectIds())
        self.failUnless(stuff_folder_id in self.blog.objectIds())
        self.failUnless(images_folder_id in self.blog.objectIds())

    def test_010catalogqueries(self):
        #Test for catalog queries
        self.makeEntries()
        #Recent entries
        entries = self.blog.getRecentEntry(type=1,limit=5,full_objects=True)
        test_eids = ['entry_%d' % x for x in (2,1,0)]
        self.failUnless([x.id for x in entries] == test_eids,
                            'Wrong order for entry limit in date')

        entries = self.blog.getRecentEntry(type=2,limit=5,full_objects=True)
        test_eids = ['entry_%d' % x for x in (2,1,0,3,4)]
        self.failUnless([x.id for x in entries] == test_eids,
                            'Wrong order for entry limit in count')

        #Entries in month
        entries = self.blog.getEntryInDate(year=2004,month=10,full_objects=True)
        test_eids = ['entry_%d' % x for x in (0,1,2)]
        self.failUnless([x.id for x in entries] == test_eids,
                            'Wrong order for entry in month')

        #Entries in date
        entries = self.blog.getEntryInDate(year=2004,month=9,day=15,
                                                    full_objects=True)
        test_eids = ['entry_%d' % x for x in (3,)]
        self.failUnless([x.id for x in entries] == test_eids,
                            'Wrong order for entry in month')

        #Previous entry
        entries = self.blog.getNearestEntry(base_date='2004/9/15')
        self.failUnless(entries[1].id == 'entry_4',
                            'Wrong entry,previous')

        #Next entry
        entries = self.blog.getNearestEntry(base_date='2004/9/15',
                                            sort_order='normal')
        self.failUnless(entries[1].id == 'entry_0',
                            'Wrong entry,next')

        #Entries in category
        cfold = self.blog.getCategoryFolder()
        entries = self.blog.getEntryInCategory(\
                        category_ids=[cfold['cat1'].getInternal_id(),])
        self.failUnless(len(entries[0]) != 2,
                            'Wrong entry number in cat1')
        self.failUnless(entries[0].id == 'entry_4',
                            'Wrong entry in cat1')
        self.failUnless(entries[1].id == 'entry_0',
                            'Wrong entry in cat1')

klasses.append(COREBlog2FolderTest)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    for klass in klasses:
        suite.addTest(makeSuite(klass))
    return suite

if __name__ == '__main__':
    framework()



## Controller Python Script "resetCookie"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Receive trackback
##

# returns "one" enclosed object:

type_priorities = {'ATAudio':1,
                   'ATVideo':2,
                   'File':3,
                  }

eobj = None
pri = 10000

for obj in context.getRefs('relatesTo'):
    t = obj.portal_type
    if t in type_priorities.keys() and type_priorities[t] < pri:
        eobj = obj
        pri = type_priorities[t]

return eobj




#permissions.py
#
#$Id: permissions.py 136 2005-12-21 08:42:59Z ats $

from Products.CMFCore.CMFCorePermissions import setDefaultRoles
import Products.Archetypes.public as atapi
import config

def initialize():
    permissions = {}
    types = atapi.listTypes(config.PROJECTNAME)
    for atype in  types:
        permission = "%s: Add %s" % (config.PROJECTNAME, atype['portal_type'])
        permissions[atype['portal_type']] = permission
        setDefaultRoles(permission, ('Manager','Owner'))

    return permissions

setDefaultRoles( 'COREBlog2: Add COREBlogComment',
                            ( 'Anonymous', 'Manager', 'Owner' ) )
setDefaultRoles( 'COREBlog2: Add COREBlogTrackback',
                            ( 'Anonymous', 'Manager', 'Owner' ) )




##############################################################################
#
# test_coreblogcategory.py
# Testcase Class for COREBlog2 Category
#
# Copyright (c) 2006 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: test_coreblogcategory.py 190 2006-04-04 11:33:06Z ats $
#
##############################################################################

"""COREBlog2 Category tests
"""

__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

from Testing import ZopeTestCase
from DateTime import DateTime

from Products.COREBlog2.config import comment_folder_id,catetory_folder_id,\
                stuff_folder_id,images_folder_id

from Products.COREBlog2.content.coreblogentry import COREBlogEntry


from coreblog2testcase import COREBlog2TestCase
klasses = []

class COREBlog2CategoryTest(COREBlog2TestCase): 

    def test_category_000names(self):
        #Test for standard folders existance
        self.makeEntries()
        cat_names = [ 'cat%d' % x for x in range(1,10) ]
        catf = self.blog.getCategoryFolder()
        cnt = 0
        for n in cat_names:
            self.failUnless(hasattr(catf,n))
            c = catf[n]
            self.failUnless(c.getInternal_id() != cnt)
            cnt += 1



klasses.append(COREBlog2CategoryTest)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    for klass in klasses:
        suite.addTest(makeSuite(klass))
    return suite

if __name__ == '__main__':
    framework()



##############################################################################
#
# COREBlog2.0
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: __init__.py 90 2005-11-25 10:37:43Z ats $
#
##############################################################################


from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory

from config import SKINS_DIR, GLOBALS, PROJECTNAME, ADD_CONTENT_PERMISSION
from permissions import initialize as initialize_permissions

import COREBlogTool

registerDirectory(SKINS_DIR, GLOBALS)

def initialize(context):
    ##Import Types here to register them
    from content import coreblog2
    from content import coreblogstufffolder
    from content import coreblogentry
    from content import coreblogcategory
    from content import coreblogcomment
    from content import coreblogtrackback

    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)

    utils.ToolInit('COREBlog Tool', tools=(COREBlogTool.COREBlog2Tool,),
            product_name=PROJECTNAME, icon='tool.gif',
            ).initialize(context)

    permissions = initialize_permissions()
    allTypes = zip(content_types, constructors)
    for atype, constructor in allTypes:
        kind = "%s: %s" % (config.PROJECTNAME, atype.archetype_name)
        utils.ContentInit(
            kind,
            content_types      = (atype,),
            permission         = permissions[atype.portal_type],
            extra_constructors = (constructor,),
            fti                = ftis,
            ).initialize(context)




##############################################################################
#
# coreblog2testcase.py
# Skelton Class for COREBlog2 Folder
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: coreblog2testcase.py 96 2005-11-27 16:38:04Z ats $
#
##############################################################################

"""COREBlog2 tests
"""

__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Acquisition import aq_base
from DateTime import DateTime

from Products.PloneTestCase import PloneTestCase

PloneTestCase.installProduct('COREBlog2')
PloneTestCase.setupPloneSite(products=['COREBlog2'])

blog_id = 'blog'

entry_body = """
Header
======

Text, text, text

* List
* List
"""

entry_data = (
(entry_body,['cat1'],'2004/10/12'), #0
(entry_body,['cat2'],'2004/10/13'), #1
(entry_body,['cat3'],'2004/10/14'), #2
(entry_body,['cat4'],'2004/9/15'), #3
(entry_body,['cat1'],'2004/8/16'), #4
(entry_body,['cat2'],'2099/1/10'), #5
)

class COREBlog2TestCase(PloneTestCase.PloneTestCase):

    def afterSetUp(self):
        # Create a COREBlog2 folder
        self.folder.invokeFactory('COREBlog2', id=blog_id)
        self.failUnless(hasattr(aq_base(self.folder), blog_id))
        self.blog = self.folder[blog_id]


    def makeEntries(self):
        #Test for adding category
        ZopeTestCase._print('Try to add categories ... \n')
        catf = self.blog.getCategoryFolder()
        cat_names = [ 'cat%d' % x for x in range(1,10) ]
        for catname in cat_names:
            catf.invokeFactory('COREBlogCategory',id=catname)
        #Test for category creation
        for catname in cat_names:
            self.failUnless(hasattr(catf, catname),
                        'Category %s not found' %catname)

        #Try to add entry
        ZopeTestCase._print('Try to add entries ... \n')
        cfold = self.blog.getCategoryFolder()
        for cnt in range(0,len(entry_data)):
            new_id = 'entry_%d' % cnt
            self.blog.invokeFactory('COREBlogEntry',new_id)
            self.failUnless(hasattr(self.blog, new_id),
                        'Entry %s not found' %catname)
            ent = self.blog[new_id]
            ent.setBody(entry_data[cnt][0],mimetype='text/structured')
            ent.setEntry_categories(\
                [str(cfold[x].getInternal_id()) for x in entry_data[cnt][1]])
            ent.setEffectiveDate(DateTime(entry_data[cnt][2]))
            ent.indexObject()


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(COREBlog2Test))
    return suite

if __name__ == '__main__':
    framework()



## Return referenced media content presentation
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=entry = None
##title=Return media size, setting for entry,'no_resize',(0,0) means no resize
##

if not entry:
    entry = container

# Sizes from ATContentType.content.image
sizes= {'large'   : (768, 768),
       'preview' : (400, 400),
       'mini'    : (200, 200),
       'thumb'   : (128, 128),
       'tile'    :  (64, 64),
       'icon'    :  (32, 32),
       'listing' :  (16, 16),
      }

sizestr = entry.getMedia_size()
if sizestr == 'no_resize' or not sizes.has_key(sizestr):
    return sizestr,0,0
else:
    return [sizestr] + list(sizes[sizestr])





## Return referenced media content presentation
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=obj
##title=Return referenced media object's presentation
##

from Products.CMFCore.utils import getToolByName

INTERFACE = 'Products.COREBlog2.interfaces.IInlineObject.IInlineObject'

itool = getToolByName(context, 'portal_interface')

try:
    if itool.objectImplements(obj, INTERFACE) and obj.isInlineObject():
        tp,macro = obj.getInlineView()
        if tp and macro:
            return container[tp].macros[macro]
except:
    pass

# Metal tempate for media handler
tp = container['media_view']
macro_keys = tp.macros.keys()
ct = ''
try:
    ct = '_' + obj.getContentType()
    ct = ct.replace('/','_')
except:
    pass

# First,see object portal_type and return appropreate template
otp = obj.portal_type
lotp = obj.portal_type.lower()

key_trials = [otp,lotp,otp + ct,lotp + ct]

for trial in key_trials:
    if trial in macro_keys:
        # The macro for object is there !
        return tp.macros[trial]

# No custom template, so return object itself...
return tp.macros['Othrers__']





##############################################################################
#
# coreblog2.py
# Class for COREBlog2 Folder
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: coreblog2.py 152 2005-12-31 11:30:31Z ats $
#$URL$
#$Rev: 152 $
#$Date: 2005-12-31 20:30:31 +0900 (土, 31 12 2005) $
#
##############################################################################

#Base classes
from Products.ATContentTypes.content.base import ATCTBTreeFolder,\
                                 ATCTMixin,updateActions,updateAliases
from Products.ATContentTypes.content.folder import ATBTreeFolder,\
                                                   ATBTreeFolderSchema,\
                                                   finalizeATCTSchema
from Products.ATContentTypes.interfaces import IATBTreeFolder
from Products.Archetypes.public import Schema,registerType
#Fields
from Products.Archetypes.public import StringField,IntegerField,\
                                   LinesField,ReferenceField,BooleanField
#Widgets
from Products.Archetypes.public import StringWidget,TextAreaWidget,\
                               IntegerWidget,SelectionWidget,BooleanWidget

from Products.Archetypes.utils import DisplayList
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.CMFPlone.migrations.migration_util import safeEditProperty

from Products.COREBlog2.config import PROJECTNAME,\
                   comment_folder_id,catetory_folder_id,\
                   stuff_folder_id,images_folder_id,default_category_id,\
                   coreblog2_meta_type,coreblogentry_meta_type,\
                   coreblogcomment_meta_type,coreblogtrackback_meta_type,\
                   coreblogcategory_meta_type,far_far_past,far_far_future
from Products.COREBlog2.content.coreblogentry\
                 import COREBlogEntry,comment_status,trackback_status

from Products.COREBlog2.configuration import zconf

from DateTime import DateTime

from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View,ListFolderContents,\
                    ModifyPortalContent
from Products.CMFCore.utils import getToolByName

import calendar
import os.path
import sys

__doc__="""Blog Product for Plone 'COREBlog:COREBlog'
$Id: coreblog2.py 152 2005-12-31 11:30:31Z ats $"""

COREBLOG2_DIR = os.path.abspath(os.path.dirname(__file__))
COREBLOG2_DIR = os.path.join(COREBLOG2_DIR,os.path.pardir)

__version__ = open(os.path.join(COREBLOG2_DIR,'version.txt')).read().strip()
__product_version__ = "COREBlog2 " + __version__

__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

COREBlog2Schema = ATBTreeFolder.schema.copy() +  Schema((

    #Basic settings
    StringField('long_description',
        searchable=1,
        widget=TextAreaWidget(label='Long description',
                    description='',
                    label_msgid='label_long_description',
                    description_msgid='help_long_description',
                    i18n_domain='plone',
                    cols=60,rows=5),
        ),

    #Entry listing
    IntegerField('top_entry_count_setting',
        searchable=0,
        default = 1,
        widget=SelectionWidget(label='Entries per page',
                    description='',
                    label_msgid='label_top_entry_count_setting',
                    description_msgid='help_top_entry_count_setting',
                    i18n_domain='plone',
                    ),
        vocabulary=DisplayList((
                    (1, 'In days','label_show_entry_in_days'),
                    (2, 'In entry count','label_show_entry_in_count'),
                    )),
        schemata='cbsettings_display',
        ),

    IntegerField('top_entry_count',
        searchable=0,
        default = zconf.coreblog2.top_entry_count_default,
        widget=IntegerWidget(label='Entries count',
                    description='',
                    label_msgid='label_top_entry_count',
                    description_msgid='help_top_entry_count',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_display',
        ),

    IntegerField('portlet_item_count',
        searchable=0,
        default = zconf.coreblog2.portlet_item_count_default,
        widget=IntegerWidget(label='Items per portlet',
                    description='',
                    label_msgid='label_portlet_item_count',
                    description_msgid='help_portlet_item_count',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_display',
        ),

    IntegerField('batch_size',
        searchable=0,
        default = zconf.coreblog2.batch_size_default,
        widget=IntegerWidget(label='Batch size',
                    description='',
                    label_msgid='label_batch_size',
                    description_msgid='help_batch_size',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_display',
        ),

    #PING/Trackback
    StringField('trackback_base',
        searchable=0,
        widget=StringWidget(label='Trackback base URL',
                    description='',
                    label_msgid='label_trackback_base',
                    description_msgid='help_trackback_base',
                    i18n_domain='plone',
                    size=60,
                    ),
        schemata='cbsettings_entry',
        ),

    StringField('ping_servers',
        searchable=1,
        widget=TextAreaWidget(label='Ping servers',
                    description='',
                    label_msgid='label_ping_servers',
                    description_msgid='help_ping_servers',
                    i18n_domain='plone',
                    cols=60,rows=5
                    ),
        schemata='cbsettings_entry',
        ),

    #Entry
    StringField('entrydate_format',
        searchable=0,
        default = '%Y/%m/%d',
        widget=StringWidget(label='Entrydate format',
                    description='',
                    label_msgid='label_entrydate_format',
                    description_msgid='help_entrydate_format',
                    i18n_domain='plone',
                    size=60,
                    ),
        schemata='cbsettings_display',
        ),

    StringField('body_default_format',
        searchable=0,
        default = 'text/html',
        widget=SelectionWidget(label='Default format for entry body',
                    description='',
                    label_msgid='label_body_default_format',
                    description_msgid='help_body_default_format',
                    i18n_domain='plone',
                    ),
        vocabulary=zconf.coreblog2.allowed_content_types ,
        schemata='cbsettings_entry',
        ),

    IntegerField('allow_comment_default',
        searchable=0,
        default = COREBlogEntry.comment_open,
        widget=SelectionWidget(label='Default comment status',
            label_msgid='label_allow_comment_default',
            i18n_domain='plone',),
        vocabulary=DisplayList(comment_status),
        schemata='cbsettings_entry',
        ),

    IntegerField('receive_trackback_default',
        searchable=0,
        default = COREBlogEntry.trackback_open,
        widget=SelectionWidget(label='Default trackback status',
            label_msgid='label_receive_trackback_default',
            i18n_domain='plone',),
        vocabulary=DisplayList(trackback_status),
        schemata='cbsettings_entry',
        ),

    BooleanField('dont_send_ping',
        searchable=1,
        default=zconf.coreblog2.dont_send_ping_default,
        widget=BooleanWidget(\
                    label='Don\'t send PING/Trackback when entry added.',
                    description='',
                    label_msgid='label_dont_send_ping',
                    description_msgid='help_dont_send_ping',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_entry',
        ),

    #Comment validation
    BooleanField('comment_require_author',
        searchable=1,
        default=True,
        widget=BooleanWidget(label='Require author',
                    description='',
                    label_msgid='label_comment_require_author',
                    description_msgid='help_comment_require_author',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_comment_trackback',
        ),

    BooleanField('comment_require_email',
        searchable=1,
        default=True,
        widget=BooleanWidget(label='Require email',
                    description='',
                    label_msgid='label_comment_require_email',
                    description_msgid='help_comment_require_email',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_comment_trackback',
        ),

    BooleanField('comment_require_url',
        searchable=1,
        default=True,
        widget=BooleanWidget(label='Require URL',
                    description='',
                    label_msgid='label_comment_require_url',
                    description_msgid='help_comment_require_url',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_comment_trackback',
        ),

    #Nortifications
    BooleanField('send_comment_notification',
        searchable=1,
        default=True,
        widget=BooleanWidget(label='Send a notification mail on new comment',
                    description='',
                    label_msgid='label_send_comment_notification',
                    description_msgid='help_send_comment_notification',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_comment_trackback',
        ),

    BooleanField('send_trackback_notification',
        searchable=1,
        default=True,
        widget=BooleanWidget(label='Send a notification mail on new trackback',
                    description='',
                    label_msgid='label_send_trackback_notification',
                    description_msgid='help_send_trackback_notification',
                    i18n_domain='plone',
                    ),
        schemata='cbsettings_comment_trackback',
        ),

    StringField('notify_from',
        searchable=1,
        widget=StringWidget(label='Notify From',
                    description='',
                    label_msgid='label_notify_from',
                    description_msgid='help_notify_from',
                    i18n_domain='plone',
                    size=60),
        schemata='cbsettings_comment_trackback',
        ),

    StringField('notify_to',
        searchable=1,
        widget=TextAreaWidget(label='Notify To',
                    description='',
                    label_msgid='label_notify_to',
                    description_msgid='help_notify_to',
                    i18n_domain='plone',
                    cols=60,rows=2),
        schemata='cbsettings_comment_trackback',
        ),


    ),
    )

#Finalize schema definition
finalizeATCTSchema(COREBlog2Schema)

class COREBlog2(ATBTreeFolder):
    """
    This is a COREBlog2 class,
    base folder for COREBlog2
    """
    
    archetype_name = "COREBlog2"
    meta_type = coreblog2_meta_type
    content_icon = 'coreblog2folder_icon.gif'
    typeDescription= 'A root folder for COREBlog2, contains blog content,  such as entry,comment,category etc.'
    typeDescMsgId  = 'coreblog2folder_description'
    global_allow   = True
    suppl_views    = ()

    schema = COREBlog2Schema

    allowed_content_types = [
            'COREBlogEntry',
            ]

    _at_rename_after_creation = True

    __implements__ = (ATCTBTreeFolder.__implements__, IATBTreeFolder)

    security = ClassSecurityInfo()

    # Set up views
    default_view = 'coreblog_view'
    immediate_view = 'coreblog_view'

    actions = updateActions(ATCTMixin,
        (\
        {
        'id': 'edit',
        'name': 'Blog settings',
        'action': 'string:${object_url}/edit',
        'permissions': (View,)
            },
        {
        'id': 'entrylisting',
        'name': 'Entries',
        'action': 'string:${object_url}/entry_listing',
        'permissions': (ListFolderContents,),
            },
        {
        'id': 'category',
        'name': 'Categories',
        'action': 'string:${object_url}/categories/',
        'permissions': (ListFolderContents,)
            },
            ))

    aliases = updateAliases(ATBTreeFolder,
        {
        'folder_contents' : 'cbfolder_contents',
        'view' : 'coreblog_view',
        'edit' : 'blogsettings_edit',
        })

    def initializeArchetype(self, **kwargs):
        ATBTreeFolder.initializeArchetype(self, **kwargs)

        # Set slot properties
        left_slots = zconf.coreblog2.left_slots_default
        safeEditProperty(self, 'left_slots', left_slots, 'lines')
        right_slots = zconf.coreblog2.right_slots_default
        safeEditProperty(self, 'right_slots', right_slots, 'lines')

        # create subfolders
        if not hasattr(self, comment_folder_id):
            # Folder for comments/trackbacks
            self.portal_types.constructContent('COREBlogCommentFolder',
                                               self,
                                               comment_folder_id,
                                               title='Comments')

        if not hasattr(self, catetory_folder_id):
            # Folder for categories
            self.portal_types.constructContent('COREBlogCategoryFolder',
                                               self,
                                               catetory_folder_id,
                                               title='Categories')
            ## Make default category
            #cat_folder = self[catetory_folder_id]
            #cat_folder.constructContent('COREBlogCategory',
            #                            self,
            #                            default_category_id,
            #                            title='Default')

        if not hasattr(self, stuff_folder_id):
            # Folder for some stuff
            self.portal_types.constructContent('Folder',
                                               self,
                                               stuff_folder_id,
                                               title='Stuff')

        if not hasattr(self, images_folder_id):
            # Folder for images
            self.portal_types.constructContent('Folder',
                                               self,
                                               images_folder_id,
                                               title='Images')
        self.indexObject()


    def canSetDefaultPage(self):
        return False

    security.declarePrivate('setLong_description')
    def setLong_description(self, value, **kwargs):
        #
        # The setter for long_description, also making copy to 'description',
        #
        self.getField('long_description').set(self, value, **kwargs)
        transformer = getToolByName(self,'portal_transforms')
        flat_desc =  transformer.convertToData('text/plain',\
                                        self.getLong_description())
        orig_len = len(flat_desc)
        flat_desc = flat_desc.replace('\n','')
        flat_desc = flat_desc.replace('\r','')
        self.setDescription(flat_desc)

    security.declareProtected(View,'blog_url')
    def blog_url(self):
        return self.absolute_url()

    security.declareProtected(View,'blog_object')
    def blog_object(self):
        return self

    #
    # Catalog queries for object listing
    #

    security.declareProtected(View,'getRecentEntry')
    def getRecentEntry(self,type=0,limit=0,\
                       sort_on='Date',sort_order='reverse',\
                       full_objects=True):
        #
        # Returns recent entries
        # for entry list on blog's top page
        #

        # Make dictionary for query arguments
        path = {}
        path['query'] = '/'.join(self.getPhysicalPath())
        path['depth'] = 1
        args = {}
        args['path'] = path
        if not type:
            type = self.getTop_entry_count_setting()
        if not limit:
            limit = self.getTop_entry_count()

        args['meta_type'] = [coreblogentry_meta_type]
        args['sort_on'] = sort_on
        args['sort_order'] = sort_order

        if type == 1:
            # If type is 1,show entries for dates.
            # So we need to find date of most recent entry.
            args['sort_limit'] = 1
            args['Date'] = (DateTime(),DateTime(far_far_past))
            args['Date_usage'] = "range:max:min"
            recent_entries = self.portal_catalog(show_inactive=False,**args)
            if recent_entries:
                recent_date = recent_entries[0].Date
            else:
                recent_date = DateTime()
            args['Date'] = (DateTime(recent_date),DateTime(recent_date)-limit-1)
            args['Date_usage'] = "range:max:min"
            del args['sort_limit']
        else:
            args['Date'] = (DateTime(),DateTime(far_far_past))
            args['Date_usage'] = "range:max:min"
            args['sort_limit'] = limit

        # Initialize list to return
        objs = self.portal_catalog(show_inactive=False,**args)

        if full_objects:
            objs = [b.getObject() for b in objs]

        return objs


    security.declareProtected(View,'getRecentEntry')
    def synContentValues(self):
        #
        # play nice with portal_syndication_tool
        #
        return self.getRecentEntry()


    security.declareProtected(View,'getEntryInDate')
    def getEntryInDate(self,year,month,day=0,\
                       batch=False,b_size=0,b_start=0,\
                       sort_on='Date',sort_order='',\
                       full_objects=True):
        #
        # Return entryes in specific month,or day
        # Used for calender portlet and 
        #

        # Make dictionary for query arguments
        path = {}
        path['query'] = '/'.join(self.getPhysicalPath())
        path['depth'] = 1
        args = {}
        args['meta_type'] = [coreblogentry_meta_type]
        args['path'] = path
        if day:
            startday = day
            endday = day
        else:
            #Find month daycount
            startday = 1
            endday = calendar.monthrange(year,month)[1]
        start_date = DateTime("%d-%d-%d 00:00:00" % (year,month,startday))
        end_date = DateTime("%d-%d-%d 23:59:59" % (year,month,endday))
        if DateTime() < end_date:
            # Hide future entries
            end_date = DateTime()
        args['Date'] = (start_date,end_date)
        args['Date_usage'] = "range:min:max"

        args['sort_on'] = sort_on
        args['sort_order'] = sort_order

        # Initialize list to return
        objs = self.portal_catalog(show_inactive=False,**args)

        if full_objects:
            objs = [b.getObject() for b in objs]

        if batch:
            from Products.CMFPlone import Batch
            batch = Batch(objs, b_size, int(b_start), orphan=0)
            return batch

        return objs

    security.declareProtected(View,'getEntryForCalendar')
    def getEntryForCalendar(self, month, year):
        #
        # Returns entries on month,
        # For every days in calendar,if entry exists,
        # store data for mapping following...
        # {'day': #, 'entry': None}
        #
        year=int(year)
        month=int(month)

        daysByWeek=calendar.monthcalendar(year, month)
        weeks=[]

        enteirs_in_month=self.getEntryInDate(year,month,full_objects=False)
        entries = {}
        for entry in enteirs_in_month:
            day = DateTime(entry.Date).day()
            entries[day] = 1

        for week in daysByWeek:
            days=[]
            for day in week:
                if entries.has_key(day):
                    days.append({'day':day,'entry':1})
                else:
                    days.append({'day':day,'entry':None})

            weeks.append(days)

        return weeks

    security.declareProtected(View,'getNearestEntry')
    def getNearestEntry(self,base_date,sort_order='reverse',\
                        sort_on='Date',full_objects=True):
        #
        # Returns nearest entries
        #

        # Make dictionary for query arguments
        path = {}
        path['query'] = '/'.join(self.getPhysicalPath())
        path['depth'] = 1
        args = {}

        args['path'] = path
        args['meta_type'] = [coreblogentry_meta_type]
        args['sort_on'] = sort_on
        args['sort_order'] = sort_order

        args['sort_limit'] = 2
        if sort_order == 'reverse':
            args['Date'] = (DateTime(base_date),DateTime(far_far_past))
            args['Date_usage'] = "range:max:min"
        else:
            args['Date'] = (DateTime(far_far_future),DateTime(base_date))
            args['Date_usage'] = "range:max:min"

        # Initialize list to return
        objs = self.portal_catalog(show_inactive=False,**args)

        if full_objects:
            objs = [b.getObject() for b in objs]

        return objs

    security.declarePrivate('getRecentItem')
    def getRecentItem(self,limit=0,meta_types=[],\
                        sort_on='Date',sort_order='',full_objects=True):
        # Make dictionary for query arguments
        path = {}
        path['query'] = '/'.join(self.getCommentFolder().getPhysicalPath())
        path['depth'] = 1
        args = {}
        args['path'] = path
        args['Date'] = (DateTime(),DateTime(far_far_past))
        args['Date_usage'] = "range:max:min"
        if not limit:
            limit = self.getPortlet_item_count()

        args['meta_type'] = meta_types
        args['sort_on'] = sort_on
        args['sort_order'] = sort_order

        args['sort_limit'] = limit

        # Initialize list to return
        objs = self.portal_catalog(show_inactive=False,**args)

        if full_objects:
            objs = [b.getObject() for b in objs]

        return objs


    security.declareProtected(View,'getRecentComment')
    def getRecentComment(self,limit=0,\
                       sort_on='Date',sort_order='reverse',\
                       full_objects=True):
        #
        # Returns recent comments
        #
        return self.getRecentItem(meta_types=[coreblogcomment_meta_type],\
                                  sort_on=sort_on,sort_order=sort_order,\
                                  full_objects=full_objects)


    security.declareProtected(View,'getRecentTrackback')
    def getRecentTrackback(self,limit=0,\
                       sort_on='Date',sort_order='reverse',\
                       full_objects=True):
        #
        # Returns recent trackback
        #
        return self.getRecentItem(meta_types=[coreblogtrackback_meta_type],\
                                  sort_on=sort_on,sort_order=sort_order,\
                                  full_objects=full_objects)


    #
    # Accessor for subfolders
    #

    security.declareProtected(View,'getCategoryFolder')
    def getCategoryFolder(self):
        """
        Return Category Folder
        """
        return self[catetory_folder_id]

    security.declareProtected(View,'getCommentFolder')
    def getCommentFolder(self):
        """
        Return Comment Folder
        """
        return self[comment_folder_id]

    #
    # Category management
    #

    security.declareProtected(View,'getEntryInCategory')
    def getEntryInCategory(self,category_ids,\
                       sort_on='Date',sort_order='',\
                       batch=False,b_size=0,b_start=0,\
                       full_objects=True):
        #
        # Return entryes in specific category(s)
        #

        # Make dictionary for query arguments
        path = {}
        path['query'] = '/'.join(self.getPhysicalPath())
        path['depth'] = 1
        args = {}
        args['path'] = path
        args['meta_type'] = [coreblogentry_meta_type]
        category_ids = [str(b) for b in category_ids]
        args['getEntry_categories'] = category_ids

        args['meta_type'] = [coreblogentry_meta_type]
        args['sort_on'] = sort_on
        args['sort_order'] = sort_order

        # Initialize list to return
        objs = self.portal_catalog(show_inactive=False,**args)

        if full_objects:
            objs = [b.getObject() for b in objs]

        if batch:
            from Products.CMFPlone import Batch
            batch = Batch(objs,b_size, int(b_start), orphan=0)
            return batch

        return objs


    security.declareProtected(View, 'getCategoryMap')
    def getCategoryMap(self,full_objects=True):
        #Returns category mapping
        cat_d = {}
        catfolder = self.getCategoryFolder()
        if full_objects:
            # Infrate real object
            for cat_obj in catfolder.objectValues(coreblogcategory_meta_type):
                cat_d[str(cat_obj.getInternal_id())] = \
                        {'title':cat_obj.title,'id':cat_obj.id,
                         'url':cat_obj.absolute_url(),
                         'obj':cat_obj }
        else:
            path = {}
            path['query'] = '/'.join(catfolder.getPhysicalPath())
            path['depth'] = 1
            contentFilter = {'path':path,\
                             'portal_type':coreblogcategory_meta_type}
            for cat_obj in \
                self.portal_catalog.queryCatalog(contentFilter,show_all=1):
                cat_d[str(cat_obj.getInternal_id)] = \
                        {'title':cat_obj.title,'id':cat_obj.id,
                         'url':catfolder.absolute_url() + '/' + cat_obj.id}
        return cat_d


    security.declareProtected(View, 'getCategoryById')
    def getCategoryById(self,id,full_objects=True):
        #Returns category by internal ID
        cat_folder = self[catetory_folder_id]
        path = {}
        path['query'] = '/'.join(cat_folder.getPhysicalPath())
        path['depth'] = 1
        contentFilter = {'path':path,'portal_type':coreblogcategory_meta_type}
        for cat_obj in \
            self.portal_catalog.queryCatalog(contentFilter,show_all=1):
            if str(cat_obj.getInternal_id) == str(id):
                if full_objects:
                    return cat_obj.getObject()
                else:
                    return cat_obj
        return None

    #
    # Sending PING
    #
    security.declareProtected(ModifyPortalContent, 'sendPing')
    def sendPing(self,idx = -1):
        """
        Send ping to servers on setting.
        If any idx served, only one ping will be sent.
        If idx == -1, try to send all ping servers.
        """
        cbtool = getToolByName(self,'coreblog2_tool')
        blogurl = self.absolute_url()
        if self.getTrackback_base():
            blogurl = self.getTrackback_base()
        svs = self.getPing_servers().split('\n')
        indexlist = range(0,len(svs))
        if idx != -1:
            indexlist = [index]
        rl = []
        for i in indexlist:
            if svs[i].find('http:') == 0:
                r = cbtool.sendPing(svs[i],self.title,blogurl,
                                __product_version__)
                rl.append(r)
        return rl

    #
    # Misc methods
    #

    security.declareProtected(View, 'getEntryFieldOrder')
    def getEntryFieldOrder(self):
        #
        #Returns edit field order for entry
        #
        return ['id','title','subtitle','entry_categories','body','extend',
                'allow_comment','receive_trackback','relatedItems',
                'media_position','media_size']

registerType(COREBlog2,PROJECTNAME)




#  ZConfig loader for COREBlog2,
#  based on ATContentTypes
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU Gefneral Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""COREBlog2 ZConfig schema loader

"""
__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

import os

from ZConfig.datatypes import Registry
from ZConfig.loader import SchemaLoader
from Products.COREBlog2.configuration import datatype

# schema file
DIR = os.path.dirname(os.path.abspath(__file__))
SCHEMA_FILE_NAME = 'schema.xml'
SCHEMA_FILE = os.path.join(DIR, SCHEMA_FILE_NAME)
print SCHEMA_FILE

# schema
coreblog2Schema = None
def loadSchema(file, registry=Registry(), overwrite=False):
    """Loads a schema file
    
    * file
      A path to a file
    * registry
      A ZConfig datatypes registry instance
    * overwrite
      Overwriting the existing global schema is not possible unless overwrite
      is set to true. Useful only for unit testing.
    """
    global coreblog2Schema
    if coreblog2Schema is not None and not overwrite:
        raise RuntimeError, 'Schema is already loaded'
    schemaLoader = SchemaLoader(registry=registry)
    coreblog2Schema = schemaLoader.loadURL(file)
    return coreblog2Schema

loadSchema(SCHEMA_FILE)

__all__ = ('coreblog2Schema',)



##############################################################################
#
# coreblogentry.py
# Class for COREBlog2 Entry
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: coreblogentry.py 164 2006-01-25 06:54:56Z ats $
#
##############################################################################

#Base classes
from Products.ATContentTypes.content.base import ATCTContent,\
                 ATCTMixin,updateActions,updateAliases,translateMimetypeAlias
from Products.ATContentTypes.content.schemata import ATContentTypeSchema,\
                                                     finalizeATCTSchema
from Products.ATContentTypes.interfaces import IATBTreeFolder
from Products.Archetypes.public import Schema,registerType
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin

from Products.ATContentTypes.interfaces import IATDocument
#Fields
from Products.Archetypes.public import StringField,TextField,IntegerField,\
                                       ReferenceField,LinesField
#Widgets
from Products.Archetypes.public import StringWidget,TextAreaWidget,\
                                       IntegerWidget,RichWidget,\
                                       ReferenceWidget,SelectionWidget,\
                                       IdWidget

from Products.Archetypes.utils import DisplayList
from Products.Archetypes.public import AnnotationStorage
#Marshaller
from Products.Archetypes.public import RFC822Marshaller
#Custom widget
from Products.COREBlog2.COREBlogWidgets import CategoryWidget

from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import StringField,IntegerField
from Products.Archetypes.public import StringWidget
from Products.Archetypes.public import BaseContent, registerType
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
from Products.COREBlog2.config import PROJECTNAME,\
                  comemnt_rel_id,trackback_rel_id,\
                  coreblogentry_meta_type,coreblogcomment_meta_type,\
                  coreblogtrackback_meta_type

from Products.COREBlog2.configuration import zconf

from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View,ModifyPortalContent

from sets import Set

description_length = zconf.coreblog2.description_length

comment_status = (  (1, "None(cannot add,hidden)",
                                "label_entrycommentstatus_none"),
                    (2, "Open(can add,shown)",
                                "label_entrycommentstatus_open"),
                    (3, "Closed(cannot add,shown)",
                                "label_entrycommentstatus_closed"),
                    )

trackback_status = ((1, "None(cannot add,hidden)",
                         "label_entrycommentstatus_none"),
                    (2, "Open(can add,shown)",
                        "label_entrycommentstatus_open"),
                    (3, "Closed(cannot add,shown)",
                        "label_entrycommentstatus_closed"),
                    )

COREBlogEntrySchema = ATContentTypeSchema.copy() +  Schema((

    StringField('id',
        required=True,
        widget=IdWidget(label='ID',
            description='',
            label_msgid='label_entry_id',
            description_msgid='help_entry_id',
            i18n_domain='plone',
            size=60),
        ),

    #Entry contents
    StringField('subtitle',
        searchable=True,
        index='TextIndex',
        widget=StringWidget(label='Subtitle',
            description='',
            label_msgid='label_subtitle',
            description_msgid='help_subtitle',
            i18n_domain='plone',
            size=60),
        ),

    TextField('body',
        required=True,
        primary=True,
        searchable=True,
        index='TextIndex',
        default_output_type='text/html',
        default_content_type='text/html',
        allowable_content_types = zconf.coreblog2.allowed_content_types ,
        widget=RichWidget(label='Body',
            description='',
            label_msgid='label_body',
            description_msgid='help_body',
            i18n_domain='plone',
            allow_file_upload=False,
            cols=40,rows=30),
        ),

    TextField('extend',
        searchable=True,
        index='TextIndex',
        default_output_type='text/html',
        default_content_type='text/html',
        allowable_content_types = zconf.coreblog2.allowed_content_types ,
        widget=RichWidget(label='Extend',
            description='',
            label_msgid='label_extend',
            description_msgid='help_extend',
            i18n_domain='plone',
            allow_file_upload=False,
            cols=40,rows=5),
        schemata='cbentry_extented_fields',
        ),

    #Category
    LinesField('entry_categories',
        required=True,
        widget=CategoryWidget(label='Category(s)',
            description='',
            label_msgid='label_entry_categories',
            description_msgid='help_entry_categories',
            i18n_domain='plone',),
       index='KeywordIndex',
        ),

    #Comments
    ReferenceField('entry_comments',
        multivalued=True,
        relationship=comemnt_rel_id,
        allowed_types=(coreblogcomment_meta_type,),
        widget=ReferenceWidget(label='Comment references',
            label_msgid='label_entry_comments',
            description_msgid='help_entry_comments',
            visible={'view':'invisible','edit':'hidden'},
            i18n_domain='plone',),
        ),

    #Trackbacks
    ReferenceField('entry_trackbacks',
        multivalued=True,
        relationship=trackback_rel_id,
        allowed_types=(coreblogtrackback_meta_type,),
        ),

    TextField('trackback_url',
        searchable=0,
        default_output_type='text/plain',
        widget=TextAreaWidget(label='Trackback',
            description='',
            label_msgid='label_trackback',
            description_msgid='help_trackback',
            i18n_domain='plone',
            allow_file_upload=False,
            #visible={'view':'invisible','edit':'hidden'},
            cols=40,rows=3),
        ),

    TextField('sent_trackback_url',
        searchable=False,
        default_output_type='text/plain',
        ),

    IntegerField('allow_comment',
        searchable=0,
        default = 2,
        widget=SelectionWidget(label='Comment status',
            label_msgid='label_allow_comment',
            description_msgid='help_allow_comment',
            i18n_domain='plone',),
        vocabulary=DisplayList(comment_status),
        schemata='cbentry_extented_fields',
        ),

    IntegerField('receive_trackback',
        searchable=0,
        default = 2,
        widget=SelectionWidget(label='Trackback status',
            label_msgid='label_receive_trackback',
            description_msgid='help_receive_trackback',
            i18n_domain='plone',),
        vocabulary=DisplayList(trackback_status),
        schemata='cbentry_extented_fields',
        ),

    IntegerField('media_position',
        searchable=0,
        default = 1,
        widget=SelectionWidget(label='Media position',
                    description='',
                    label_msgid='label_media_position',
                    description_msgid='help_media_position',
                    i18n_domain='plone',),
        vocabulary=DisplayList((
                  (1, "Top,horizontal",
                      "label_media_position_top"),
                  (2, "Bottom,horizontal",
                      "label_media_position_bottom"),
                  (3, "Left,vertical",
                      "label_media_position_left"),
                  (4, "Right,vertical",
                      "label_media_position_right"),
                    ))
        ),

    StringField('media_size',
        searchable=0,
        default = 'thumb',
        widget=SelectionWidget(label='Media size',
                    description='',
                    label_msgid='label_media_size',
                    description_msgid='help_media_size',
                    i18n_domain='plone',),
        vocabulary=DisplayList((
                    ('large', "Large (768,768)",
                        "label_media_size_large"),
                    ('preview', "Prenew (400,400)",
                        "label_media_size_preview"),
                    ('mini', "Mini (200,200)",
                        "label_media_size_mini"),
                    ('thumb', "Thumbnail (128,128)",
                        "label_media_size_thumbnail"),
                    ('no_resize', "No resize",
                        "label_media_size_noresize"),
                    ))
        ),

    ), marshall=RFC822Marshaller()
    )

finalizeATCTSchema(COREBlogEntrySchema)

class COREBlogEntry(ATCTContent):
    """
    This is an Entry class for coreblog2
    """
    
    schema = COREBlogEntrySchema
    archetype_name = "COREBlog Entry"
    meta_type = coreblogentry_meta_type
    typeDescription= 'A Entry for COREBlog2.'
    typeDescMsgId  = 'coreblog2entry_description'
    content_icon   = 'coreblogentry_icon.gif'
    # Not to be shown at add item menu
    global_allow = False
    allow_discussion = False
    
    __implements__ = (ATCTContent.__implements__,
                      IATDocument,
                      HistoryAwareMixin.__implements__,
                     )
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    # Set up views
    default_view = 'cbentry_view'
    immediate_view = 'cbentry_view'

    _at_rename_after_creation = True

    actions = updateActions(ATCTMixin,({
        'id': 'comments',
        'name': 'Comments',
        'action': 'string:${object_url}/cbentry_comments',
        'permissions': (CMFCorePermissions.ModifyPortalContent,),
            },
               {
        'id': 'trackbacks',
        'name': 'Trackbacks',
        'action': 'string:${object_url}/cbentry_trackbacks',
        'permissions': (CMFCorePermissions.ModifyPortalContent,),
            },
            ))

    aliases = updateAliases(ATCTContent,
        {
        'edit' : 'cbentry_edit',
        'view' : 'cbentry_view'
        })

    # Not to be shown at navigation
    exclude_from_nav = True

    # Constants
    comment_none = 1
    comment_open = 2
    comment_closed = 3

    trackback_none = 1
    trackback_open = 2
    trackback_closed = 3


    def initializeArchetype(self, **kwargs):
        ATCTContent.initializeArchetype(self, **kwargs)
        self._ping_sent = False

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        #
        # Hook method, called before object deletion
        #

        # Get list for ids of referenced comments,trackbacks
        ids = [obj.id for obj in self.getComment()]
        ids += [obj.id for obj in self.getTrackback()]

        # Now delete them all!
        self.aq_parent.getCommentFolder().manage_delObjects(ids)

        # Call superclass method
        ATCTContent.manage_beforeDelete(self,item, container)


    security.declareProtected(ModifyPortalContent, 'setBody')
    def setBody(self, value, **kwargs):
        #
        # The setter for body, also making copy to 'description',
        # and store 'flat_descriptin' for trackback auto discoverty
        #
        self.getField('body').set(self, value, **kwargs)
        transformer = getToolByName(self,'portal_transforms')
        flat_desc =  transformer.convertToData('text/plain',self.getBody())
        orig_len = len(flat_desc)
        flat_desc = unicode(flat_desc,'utf-8','ignore')
        flat_desc = flat_desc[:description_length].encode('utf-8')
        flat_desc = flat_desc.replace('\n','')
        flat_desc = flat_desc.replace('\r','')
        if orig_len > description_length:
            flat_desc += '...'
        self.setDescription(flat_desc)

    #
    # Codes for CMF compatibility
    #

    security.declareProtected(View, 'CookedBody')
    def CookedBody(self, stx_level='ignored'):
        """
        Method for CMF compatibility, returns rendered body
        """
        return self.getBody()


    security.declareProtected(ModifyPortalContent, 'EditableBody')
    def EditableBody(self):
        """
        Method for CMF compatibility, returns raw body
        """
        return self.getRawBody()

    security.declareProtected(ModifyPortalContent,
                              'setFormat')
    def setFormat(self, value):
        """
        Method for CMF compatibility, set for body format
        """
        if not value:
            value = 'text/html'
        else:
            value = translateMimetypeAlias(value)
        ATCTContent.setFormat(self, value)


    security.declareProtected(View, 'getObjSize')
    def getObjSize(self):
        return len(self.getBody() + self.getExtend())


    security.declarePrivate('_renameAfterCreation')
    def _renameAfterCreation(self, check_auto_id=False):
        """
        Renames an object like its normalized title.
        Overridden method
        """

        if check_auto_id and not self._isIDAutoGenerated(self.getId()):
            # No auto generated id
            return False

        # Check if hook method is there
        if hasattr(self,'getEntryId'):
            new_id = self.getEntryId()
        if new_id:
            invalid_id = False
            check_id = getattr(self, 'check_id', None)
            if check_id is not None:
                invalid_id = check_id(new_id, required=1)
            else:
                # If check_id is not available just look for conflicting ids
                parent = aq_parent(aq_inner(self))
                invalid_id = new_id in parent.objectIds()

            if not invalid_id:
                # Can't rename without a subtransaction commit when using
                # portal_factory!
                get_transaction().commit(1)
                self.setId(new_id)
                return new_id

        return BaseContent._renameAfterCreation(self, check_auto_id)


    #
    # Accessor to prev. next entry
    #

    security.declareProtected(View, 'getNextEntry')
    def getNextEntry(self,full_objects=True):
        """ """
        #
        # Returns next entry, according to date
        #
        parent = self.aq_parent.getCommentFolder()
        ent_list = parent.getNearestEntry(sort_order='normal',\
                        base_date=self.Date(),full_objects=full_objects)
        if len(ent_list) > 1:
            return ent_list[1]
        return None

    security.declareProtected(View, 'getPreviousEntry')
    def getPreviousEntry(self,full_objects=True):
        """ """
        #
        # Returns previous entry, according to date
        #
        parent = self.aq_parent.getCommentFolder()
        ent_list = parent.getNearestEntry(sort_order='reverse',\
                        base_date=self.Date(),full_objects=full_objects)
        if len(ent_list) > 1:
            return ent_list[1]
        return None

    #
    # Comment Management
    #

    security.declareProtected(View, 'getComment')
    def getComment(self,full_objects=True):
        #Returns entry comments

        #Get refecences comment
        coms = self.getRefs(comemnt_rel_id)
        obj_list = []

        #Set content filter
        path = {}
        path['depth'] = 0
        contentFilter = {'path':path,'portal_type':coreblogcomment_meta_type}
        for com in coms:
            path['query'] = '/'.join(com.getPhysicalPath())
            #Tell catalog if comment will be shown in this context
            for cat_obj in \
                self.queryCatalog(contentFilter,show_all=1):
                if full_objects:
                    #Append comment object itself
                    obj_list.append(cat_obj.getObject())
                else:
                    #Append catalog data
                    obj_list.append(cat_obj)
        if full_objects:
            obj_list.sort(lambda x,y: cmp(x.Date(),y.Date()))
        else:
            obj_list.sort(lambda x,y: cmp(x.Date,y.Date))

        return obj_list

    security.declareProtected(View, 'countComment')
    def countComment(self):
        #Returns entry comment count

        #Get refecences comment
        coms = self.getRefs(comemnt_rel_id)
        cnt = 0

        #Set content filter
        path = {}
        path['depth'] = 0
        contentFilter = {'path':path,'portal_type':coreblogcomment_meta_type}
        for com in coms:
            #Tell catalog if comment will be shown in this context
            path['query'] = \
                '/'.join(com.getPhysicalPath())
            if self.queryCatalog(contentFilter,show_all=False):
                cnt += 1
        return cnt

    security.declareProtected(View, 'addComment2Entry')
    def addComment2Entry(self,author='',email='',url='',title='',
                         body='',post_ip='',id='',REQUEST=None):
        #Add comment and reference to entry
        #Further feature is gurded by Zope Security,
        #so some settings in security tab should be done
        #for anonymous comment or something.

        # Check for comment status
        if self.getAllow_comment() != self.comment_open:
            return None

        type_name=coreblogcomment_meta_type
        parent = self.aq_parent.getCommentFolder()
        # Call factory method to create comment object
        if not id:
            id = parent.generateUniqueId()
        new_id = parent.invokeFactory(id=id, type_name=type_name)
        if new_id is None or new_id == '':
           new_id = id

        # Get new comment object
        com = parent[id]

        if not post_ip and REQUEST and REQUEST.has_key('REMOTE_ADDR'):
            post_ip = REQUEST['REMOTE_ADDR']

        # Set attributes
        com.setAuthor(author)
        com.setEmail(email)
        com.setUrl(url)
        com.setTitle(title)
        com.setBody(body)
        com.setPost_ip(post_ip)
        com.setEffectiveDate('')

        # Make sure to be catalloged
        com.indexObject()

        # Add reference
        self.addReference(com,comemnt_rel_id)
        return new_id


    #
    # PING Management
    #
    security.declareProtected(ModifyPortalContent, 'setBody')
    def setPingStatus(self,state=True):
        self._ping_sent = state

    security.declareProtected(View, 'hasPingSent')
    def hasPingSent(self):
        try:
            return self._ping_sent
        except:
            self._ping_sent = True
        return True


    #
    # Trackback Management(Receiving)
    #

    security.declareProtected(View, 'getTrackback')
    def getTrackback(self,full_objects=True):
        #Get refecences trackback
        tbs = self.getRefs(trackback_rel_id)
        obj_list = []

        #Set content filter
        path = {}
        path['depth'] = 0
        contentFilter = {'path':path,'portal_type':coreblogtrackback_meta_type}
        for tb in tbs:
            path['query'] = \
                '/'.join(tb.getPhysicalPath())
            #Tell catalog if trackback will be shown in this context
            for tb_obj in \
                self.queryCatalog(contentFilter,show_all=1):
                if full_objects:
                    #Append trackback object itself
                    obj_list.append(tb_obj.getObject())
                else:
                    #Append catalog data
                    obj_list.append(tb_obj)
        if full_objects:
            obj_list.sort(lambda x,y: cmp(x.Date(),y.Date()))
        else:
            obj_list.sort(lambda x,y: cmp(x.Date,y.Date))

        return obj_list

    security.declareProtected(View, 'countTrackback')
    def countTrackback(self):
        #Returns entry trackback count

        tbs = self.getRefs(trackback_rel_id)
        cnt = 0
        path = {}
        path['depth'] = 0
        contentFilter = {'path':path}
        for tb in tbs:
            path['query'] = \
                '/'.join(tb.getPhysicalPath())
            if self.queryCatalog(contentFilter,show_all=1):
                cnt += 1
        return cnt

    security.declareProtected(View, 'addTrackback2Entry')
    def addTrackback2Entry(self,title='',url='',blog_name='',
                         excerpt='',post_ip='',id='',REQUEST=None):
        #Add trackback and reference to entry
        #Further feature is gurded by Zope Security,
        #so some settings in security tab should be done
        #for anonymous trackback or something.

        # Check for trackback status
        if self.getReceive_trackback() != self.trackback_open:
            return None

        type_name=coreblogtrackback_meta_type
        parent = self.aq_parent.getCommentFolder()
        # Call factory method to create trackback object
        if not id:
            id = parent.generateUniqueId()
        new_id = parent.invokeFactory(id=id, type_name=type_name)
        if new_id is None or new_id == '':
           new_id = id

        # Get new trackback object
        tb = parent[id]

        if not post_ip and REQUEST and REQUEST.has_key('REMOTE_ADDR'):
            post_ip = REQUEST['REMOTE_ADDR']

        # Set attributes
        tb.setTitle(title)
        tb.setUrl(url)
        tb.setBlog_name(blog_name)
        tb.setExcerpt(excerpt)
        tb.setPost_ip(post_ip)

        # Make sure to be catalloged
        tb.indexObject()

        # Add reference
        self.addReference(tb,trackback_rel_id)
        return new_id


    #
    # Trackback Management(Sending)
    #
    security.declareProtected(ModifyPortalContent, 'getUnsentTrackbacks')
    def getUnsentTrackbacks(self):
        """
        Find if there is unsent trackback, if there, return true.
        """
        tobesent = Set(self.getTrackback_url().split('\n'))
        alreadysent = Set(self.getSent_trackback_url().split('\n'))
        # Get unique unsent trackback url(s)
        unsent = tobesent - alreadysent
        return list(unsent)

    security.declareProtected(ModifyPortalContent, 'sendTrackbacks')
    def sendTrackbacks(self,idx = -1):
        """
        Send trackback.
        If any idx served, only one trackback will be sent.
        If idx == -1, try to send all trackback.
        """
        cbtool = getToolByName(self,'coreblog2_tool')

        tburls = self.getUnsentTrackbacks()
        idxlist = range(0,len(tburls))
        if idx != -1:
            idxlist = [idx]
        senttbs = []
        entry_url = self.absolute_url()
        if self.aq_parent.getTrackback_base():
            repbase = self.aq_parent.absolute_url()
            entry_url = entry_url.replace(repbase,\
                            self.aq_parent.getTrackback_base())

        blog_title = self.aq_parent.title
        rls = []
        successfully_sent = []
        for i in idxlist:
            tb = tburls[i]
            if tb:
                try:
                    r = cbtool.sendTrackback(tb,self.title,entry_url,
                                    blog_title,self.Description())
                    rls.append(r)
                    if len(r) >= 2 and r[0] == 0:
                        # Trackback has been added successfully
                        successfully_sent.append(tb)
                except:
                    pass
        [ tburls.remove(x) for x in successfully_sent ]
        self.setTrackback_url('\n'.join(tburls))
        self.setSent_trackback_url('\n'.join(successfully_sent))
        return rls

registerType(COREBlogEntry, PROJECTNAME)



##############################################################################
#
# coreblogstufffolder.py
# Classes for COREBlog2 Folder for Comments/Trackbacks - Categories
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: coreblogstufffolder.py 132 2005-12-21 03:25:32Z ats $
#
##############################################################################

#Base classes
from Products.ATContentTypes.content.base import ATCTBTreeFolder,ATCTFolder,\
                                                    updateAliases
from Products.ATContentTypes.content.folder import ATBTreeFolder,ATFolder,\
                             ATBTreeFolderSchema,ATFolderSchema,\
                             finalizeATCTSchema
from Products.ATContentTypes.interfaces import IATBTreeFolder,IATFolder
from Products.Archetypes.public import Schema,registerType

#Fields
from Products.Archetypes.public import StringField,IntegerField,\
                                       LinesField,ReferenceField
#Widgets
from Products.Archetypes.public import StringWidget,TextAreaWidget,\
                                       IntegerWidget,SelectionWidget

from Products.Archetypes.utils import DisplayList
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.CMFCore import CMFCorePermissions
from Products.COREBlog2.config import PROJECTNAME

from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View

__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

COREBlogCommentFolderSchema = ATBTreeFolder.schema.copy()
COREBlogCategoryFolderSchema = ATBTreeFolder.schema.copy()

#Finalize schema definition
finalizeATCTSchema(COREBlogCommentFolderSchema)
finalizeATCTSchema(COREBlogCategoryFolderSchema)


class COREBlogCommentFolder(ATBTreeFolder):
    """
    This is a COREBlogCommentFolder class
    """
    
    archetype_name = "COREBlog Comment Folder"
    meta_type = 'COREBlogCommentFolder'
    #content_icon = 'COREBlog2.gif'
    typeDescription= 'A Folder who contains comment,trackback'
    typeDescMsgId  = 'coreblogcomment_description_edit'
    
    global_allow = False
    exclude_from_nav = True
    suppl_views    = ()
    
    allowed_content_types = (
            'COREBlogComment',
            'COREBlogTrackback',
            )
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    
    # Set up views
    default_view = 'cbcommentfolder_view'
    immediate_view = 'cbcommentfolder_view'
    
    __implements__ = (ATCTBTreeFolder.__implements__, IATBTreeFolder)
    
    aliases = updateAliases(ATBTreeFolder,
        {
        'view' : 'cbcommentfolder_view',
        })

    def canSetDefaultPage(self):
        return False

registerType(COREBlogCommentFolder, PROJECTNAME)

class COREBlogCategoryFolder(ATFolder):
    """
    This is a COREBlogCategoryFolder class
    """
    
    archetype_name = "COREBlog Category Folder"
    meta_type = 'COREBlogCategoryFolder'
    #content_icon = 'COREBlog2.gif'
    typeDescription= 'A Folder who contains category'
    typeDescMsgId  = 'coreblogcategory_description_edit'
    
    global_allow = False
    exclude_from_nav = True
    suppl_views    = ()
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    
    __implements__ = ATFolder.__implements__
    
    allowed_content_types = (
            'COREBlogCategory',
            )
    
    def canSetDefaultPage(self):
        return False

registerType(COREBlogCategoryFolder, PROJECTNAME)




##############################################################################
#
# COREBlogFolder.py
# Class for COREBlog2 Folder
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: COREBlogTool.py 156 2006-01-03 20:49:37Z ats $
#
##############################################################################

from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import log

from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from Globals import InitializeClass

from Products.COREBlog2.configuration import zconf

#Python modules
import urllib
import string,urllib,httplib,urlparse,re
from xmlrpclib import Server,Transport
import socket

timeout = zconf.coreblog2.ping_timeout_default

def safe_uencode(s):
    if isinstance(s,unicode):
        s = s.encode('utf-8')
    else:
        s = unicode(s,'utf-8').encode('utf-8')
    return s

class COREBlog2Tool(UniqueObject, SimpleItem):
    """Utility methods for COREBlog2"""

    id = 'coreblog2_tool'
    meta_type= 'COREBlog2 Tool'
    plone_tool = True

    security = ClassSecurityInfo()

    security.declarePublic('sendPing')
    def sendPing(self,
                 serverurl,
                 blogtitle,
                 url,
                 version_str = "COREBlog2",
                 char_code = "utf-8",
                 fromcode=""):
        """
           Send PING to PING server.
        """
        #blogtitle = convert_charcode(blogtitle,char_code,fromcode)
        # Sotre timeout
        oldtimeout = socket.getdefaulttimeout()
        socket.setdefaulttimeout(timeout)

        svr = Server(serverurl)
        Transport.user_agent = version_str

        try:
            resp = svr.weblogUpdates.ping(blogtitle,url)
        except Exception, e:
            resp = {'error':str(e)}

        socket.setdefaulttimeout(oldtimeout)

        return resp

    security.declarePublic('sendTrackback')
    def sendTrackback(self,
                  trackback_url,
                  title="",
                  src_url="",
                  blog_name="",
                  excerpt="",
                  agent = "COREBlog2",
                  charset="utf-8",
                  param_charset="utf-8"):
        """
        Try to send trackback request to trackback_url
        """
        try:
            #To make dictionary for POST
            params = {"title":    safe_uencode(title),
                      "url":      safe_uencode(src_url),
                      "blog_name":safe_uencode(blog_name),
                      }
            headers = ({"Content-type": "application/x-www-form-urlencoded",
                        "User-Agent": agent})
    
            if param_charset:
                params["charset"] = param_charset
            elif charset:
                headers["Content-type"] = headers["Content-type"] + \
                                                    "; charset=%s"
            # Sotre timeout
            oldtimeout = socket.getdefaulttimeout()
            socket.setdefaulttimeout(timeout)
    
            if len(excerpt) > 0:
                params["excerpt"] = safe_uencode(excerpt)
    
            tb_url_t = urlparse.urlparse(trackback_url)
            enc_params = urllib.urlencode(params)
    
            #check if trackback url contains parameter section(for PyDs!)
            ut = urlparse.urlparse(trackback_url)
            if len(ut) >= 4 and ut[4]:
                #add params to parameter section
                enc_params = ut[4] + '&' + enc_params
    
            host = tb_url_t[1]
            path = tb_url_t[2]
            con = httplib.HTTPConnection(host)
            con.request("POST", path, enc_params, headers)
            r = con.getresponse()
            http_response = r.status
            http_reason = r.reason
            resp = r.read()
            socket.setdefaulttimeout(oldtimeout)
    
            err_code_pat = re.compile("<error>(.*?)</error>",re.DOTALL)
            message_pat = re.compile("<message>(.*?)</message>",re.DOTALL)
            error_code = 0
            message = ""
            err_m = err_code_pat.search(resp)
            if err_m:
                try:
                    error_code = int(err_m.group(1))
                except:
                    pass
            else:
                error_code = 1
            mes_m = message_pat.search(resp)
            if mes_m:
                try:
                    message = mes_m.group(1)
                except:
                    pass
        except Exception,e:
            error_code = -1
            message = str(e)
            log( 'COREBlog2Tool/sendTrackback: '
                     'Some exception occured, %s' % e )
        return error_code,message

    security.declarePublic('discoverTrackback')
    def discoverTrackback(self,url):
        """
        Parse trackback:ping url from given url
        """
        
        o = urllib.urlopen(url)
        src = o.read()
        #Regex pattern
        rdfpat = re.compile("<rdf:RDF.*?</rdf:RDF>",re.DOTALL)
        #Pattern for Trackback PING URL
        tppat = re.compile("""trackback:ping="([^"]+)""",re.DOTALL)
        trackback_ping_url = ""
        #Finc Trackback PING URL
        for item in rdfpat.findall(src):
            m = tppat.search(item)
            if m:
                trackback_ping_url =  m.group(1)
                break
        
        return trackback_ping_url

    
    security.declarePublic('guess_encode')
    def guess_encode(self,s):
        """
        Guess encodings.
        possible_encodings should be set in conf file.
        """
        possible_encodings = ['utf-8','euc_jp','shift-jis','us-ascii']
        guessed_enc = 'us-ascii'
        for enc in possible_encodings:
            try:
                unicode(s,enc)
                guessed_enc = enc
                break;
            except:
                pass
        return guessed_enc

    security.declarePublic('convert_charcode')
    def convert_charcode(self,s,fromcodestr=""):
        """
        convert charcode (for Japanese)
        """
        if type(s) == type(u''):
            #unicode -> str
            s = s.encode('utf-8')
            fromcodestr = code_utf8

        retstr = s

        try:
            import pykf
            pykf_fromcodemap = { \
                pykf.EUC : "euc-jp",
                pykf.SJIS: "shift_jis",
                pykf.UTF8: "utf-8",
                pykf.JIS : "iso-2022-jp" }
            pkf_fromcode = 0
            if not fromcodestr:
                #fromcode is unknown... so do 'guess'
                fromcodestr = self.guess_encode(s)

            #Do conversion...
            retstr = unicode(s, fromcodestr, "ignore").encode('utf8')
        except:
            pass
        return retstr

    security.declarePublic('send_mail')
    def send_mail(self, body,to_addr,from_addr,subject):
        #
        # Method to bypass Zope's security.
        # For case of comment/trackback post by anonymous user
        #
        try:
            self.MailHost.send(body,to_addr, from_addr, subject)
        except Exception,e:
            log( 'COREBlog2/tbping: '
                     'Some exception occured, %s' % e )


InitializeClass(COREBlog2Tool)




##############################################################################
#
# coreblogtrackback.py
# Class for COREBlog2 Trackback
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: coreblogtrackback.py 138 2005-12-21 10:40:48Z ats $
#
##############################################################################

#Base classes
from Products.ATContentTypes.content.base import ATCTContent,updateAliases
from Products.ATContentTypes.content.schemata import ATContentTypeSchema,\
                                                     finalizeATCTSchema
from Products.ATContentTypes.interfaces import IATBTreeFolder
from Products.Archetypes.public import Schema,registerType
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin
#Fields
from Products.Archetypes.public import StringField,TextField,IntegerField
#Widgets
from Products.Archetypes.public import StringWidget,TextAreaWidget,IntegerWidget
#Marshaller
from Products.Archetypes.public import RFC822Marshaller

from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import StringField,IntegerField
from Products.Archetypes.public import StringWidget
from Products.Archetypes.public import BaseContent, registerType
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.CMFCore import CMFCorePermissions
from Products.COREBlog2.config import PROJECTNAME

from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View

COREBlogTrackbackSchema = ATContentTypeSchema.copy() +  Schema((

    #Comment content data
    StringField('url',
        searchable=1,
        widget=StringWidget(label='URL',
            description='',
            label_msgid="label_coreblog_trackback_url",
            description_msgid="help_coreblog_trackback_url",
            i18n_domain="plone",
            size=60),
        ),

    StringField('excerpt',
        searchable=1,
        primary=True,
        index='TextIndex',
        default_output_type='text/html',
        default_content_type='text/html',
        widget=StringWidget(label='excerpt',
            description='',
            label_msgid="label_coreblog_trackback_excerpt",
            description_msgid="help_coreblog_trackback_excerpt",
            i18n_domain="plone",
            size=60),
        ),

    StringField('blog_name',
        searchable=1,
        widget=StringWidget(label='blog_name',
            description='',
            label_msgid="label_coreblog_trackback_blog_name",
            description_msgid="help_coreblog_trackback_blog_name",
            i18n_domain="plone",
            size=60),
        ),

    StringField('post_ip',
        searchable=1,
        index='FieldIndex',
        widget=StringWidget(label='IP',
            description='',
            label_msgid="label_coreblog_comment_post_ip",
            description_msgid="help_coreblog_comment_post_ip",
            i18n_domain="plone",
            size=60),
        ),


    ),
    marshall=RFC822Marshaller(),
    )

finalizeATCTSchema(COREBlogTrackbackSchema)

class COREBlogTrackback(ATCTContent):
    """
    This is a Category class for COREBlog2
    """
    
    schema = COREBlogTrackbackSchema
    archetype_name = "COREBlog Trackback"
    meta_type = 'COREBlogTrackback'
    typeDescription= 'A Trackback for COREBlog2.'
    typeDescMsgId  = 'coreblog2_description_coreblog2trackback'
    content_icon = 'trackback_icon.gif'

    # Not to be shown at add item menu
    global_allow = False
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    
    _at_rename_after_creation = True
    
    # Set up views
    default_view = 'cbtrackback_view'
    immediate_view = 'cbtrackback_view'

    aliases = updateAliases(ATCTContent,
        {
        'view' : 'cbtrackback_view',
        'edit' : 'cbtrackback_edit'
        })

    # Not to be shown at navigation
    exclude_from_nav = True

    security.declareProtected(View, 'getParentEntry')
    def getParentEntry(self):
        entry = self.getBRefs()
        if entry:
            return entry[0]

registerType(COREBlogTrackback, PROJECTNAME)



##############################################################################
#
# test_coreblogentry.py
# Testcase Class for COREBlog2 Folder
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: test_coreblogentry.py 94 2005-11-27 16:37:14Z ats $
#
##############################################################################

"""COREBlog2 Entry tests
"""

__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

from Testing import ZopeTestCase
from DateTime import DateTime

from Products.COREBlog2.config import comment_folder_id,catetory_folder_id,\
                stuff_folder_id,images_folder_id

from Products.COREBlog2.content.coreblogentry import COREBlogEntry


from coreblog2testcase import COREBlog2TestCase
klasses = []

class COREBlog2EntryTest(COREBlog2TestCase): 

    def test_addcomments(self):
        #Test for standard folders existance
        self.makeEntries()
        baseentry = self.blog['entry_3']
        #Previous entry
        entry = baseentry.getPreviousEntry()
        self.failUnless(entry.id == 'entry_4',
                            'Wrong entry,previous')
        #Next entry
        entry = baseentry.getNextEntry()
        self.failUnless(entry.id == 'entry_0',
                            'Wrong entry,next')
        baseentry = self.blog['entry_2']
        entry = baseentry.getNextEntry()
        self.failUnless(entry == None,
                            'Wrong entry,next')

        #Test for add comments
        baseentry = self.blog['entry_3']
        baseentry.addComment2Entry(author='name0',email='email@com',
                    url='http://foo.bar/0',title='title0',body='body0')
        baseentry.addComment2Entry(author='name1',email='email@com',
                    url='http://foo.bar/1',title='title1',body='body1')
        self.failUnless(baseentry.countComment() == 2,
                            'Wrong comment number')

        #Try to add comment on closed entry.
        baseentry = self.blog['entry_4']
        baseentry.setAllow_comment(COREBlogEntry.comment_none)
        baseentry.addComment2Entry(author='name0',email='email@com',
                    url='http://foo.bar/0',title='title0',body='body0')
        self.failUnless(baseentry.countComment() == 0,
                            'Wrong comment number for comment setting "none"')

        baseentry.setAllow_comment(COREBlogEntry.comment_closed)
        baseentry.addComment2Entry(author='name0',email='email@com',
                    url='http://foo.bar/0',title='title0',body='body0')
        self.failUnless(baseentry.countComment() == 0,
                            'Wrong comment number for comment setting "closed"')


        #Test for add trackbacks
        baseentry = self.blog['entry_2']
        baseentry.addTrackback2Entry(blog_name='name0',url='http://foo.bar/0',
                                title='title0',excerpt='body0')
        baseentry.addTrackback2Entry(blog_name='name1',url='http://foo.bar/1',
                                title='title1',excerpt='body1')
        self.failUnless(baseentry.countTrackback() == 2,
                            'Wrong trackback number')

        #Try to add trackback on closed entry.
        baseentry = self.blog['entry_0']
        baseentry.setAllow_comment(COREBlogEntry.trackback_none)
        baseentry.addTrackback2Entry(blog_name='name0',url='http://foo.bar/0',
                                title='title0',excerpt='body0')
        self.failUnless(baseentry.countComment() == 0,
                            'Wrong trackback number for comment setting "none"')

        baseentry.setAllow_comment(COREBlogEntry.trackback_closed)
        baseentry.addTrackback2Entry(blog_name='name0',url='http://foo.bar/0',
                                title='title0',excerpt='body0')
        self.failUnless(baseentry.countComment() == 0,
                        'Wrong trackback number for comment setting "closed"')

klasses.append(COREBlog2EntryTest)

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    for klass in klasses:
        suite.addTest(makeSuite(klass))
    return suite

if __name__ == '__main__':
    framework()



from Interface import Interface, Attribute

class IInlineObject(Interface):
    """Marker interface for IInlineObject
    """

    def isInlineObject():
        """
        Return true if object is "Inline object",
        who must have "Inline view".
        Inline object will be shown in entry.
        Optionally, object might have "Attachment view",
        which used for object presentation, bottom of the entry.
        
        method must be accessed with permission CMFCorePermissions.View
        """

    def getInlineView():
        """
        Return path of metal, which define "Inline view" of object.

        method must be accessed with permission CMFCorePermissions.View
        """

    def getAttachmentView():
        """
        Return path to metal, ('template_name','macro_id'),
                        which define "Attachment view" of object.

        method must be accessed with permission CMFCorePermissions.View
        """




## Hook method to return entry id, generated from title or something
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Hook method to return entry id, generated from title or something
##

# Write some code for entry id generation.
# You need simply return string id

# Return 'None' not to use this mechanis

return None



#  ZConfig loader for COREBlog2,
#  based on ATContentTypes
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU Gefneral Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""COREBlog2 ZConfig loader

"""
__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

import os

from ZConfig.loader import ConfigLoader
from Globals import INSTANCE_HOME
from Products.COREBlog2.configuration.schema import coreblog2Schema

# directories
INSTANCE_ETC = os.path.join(INSTANCE_HOME, 'etc')
_here = os.path.dirname(__file__)
COREBLOG2_HOME = os.path.dirname(os.path.abspath(os.path.join(_here)))
COREBLOG2_ETC = os.path.join(COREBLOG2_HOME, 'etc')

# files
CONFIG_NAME = 'coreblog2.conf'
INSTANCE_CONFIG = os.path.join(INSTANCE_ETC, CONFIG_NAME)
COREBLOG2_CONFIG = os.path.join(COREBLOG2_ETC, CONFIG_NAME)
COREBLOG2_CONFIG_IN = os.path.join(COREBLOG2_ETC, CONFIG_NAME+'.in')

# check files for existence
if not os.path.isfile(INSTANCE_CONFIG):
    INSTANCE_CONFIG = None

if not os.path.isfile(COREBLOG2_CONFIG):
    COREBLOG2_CONFIG = None

if not os.path.isfile(COREBLOG2_CONFIG_IN):
    raise RuntimeError("Unable to find configuration file at %s" % 
                        COREBLOG2_CONFIG_IN)
FILES = (INSTANCE_CONFIG, COREBLOG2_CONFIG, COREBLOG2_CONFIG_IN,)

# config
zconf, handler, conf_file = None, None, None
def loadConfig(files, schema=coreblog2Schema, overwrite=False):
    """Config loader
    
    The config loader tries to load the first existing file
    """
    global zconf, handler, conf_file
    if not isinstance(files, (tuple, list)):
        files = (files, )
    if zconf is not None and not overwrite:
        raise RuntimeError, 'Configuration is already loaded'
    for file in files:
        if file is not None:
            if not os.path.exists(file):
                raise RuntimeError, '%s does not exist' % file
            conf_file = file
            zconf, handler = ConfigLoader(schema).loadURL(conf_file)
            break


loadConfig(FILES)

__all__ = ('zconf', 'handler', 'conf_file')



##############################################################################
#
# COREBlogWidgets.py
# Classes for COREBlog2 Trackback
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: COREBlogWidgets.py 133 2005-12-21 08:23:15Z ats $
#
##############################################################################

from Products.Archetypes.Widget import TypesWidget
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerWidget, registerPropertyType

class CategoryWidget(TypesWidget):
    _properties = TypesWidget._properties.copy()
    _properties.update({
        'macro' : 'cbcategory_widget',
        'folderid' : 'categories', #folder id for category
        'helper_css': ('coreblogwidgets.css',),
        })
    security = ClassSecurityInfo()

registerWidget(CategoryWidget,
               title='Category widgets',
               description="This is Category widget.",
               used_for=('Used for selecting category.',)
               )

registerPropertyType('folderid', 'string', CategoryWidget)




## Controller Python Script "resetCookie"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Reset cookie for comment
##

import string
import DateTime

#parsing modes...
singleline_mode = 0
multiline_mode = 1
subcontent_mode = 2

#separetors
ml_sep = '-'*5
ent_sep = '-'*8
subcontents = ['comment','ping']

def parse_block(lines):
    #parsing a entry and store parts on dictionary
    d = {}
    mode = singleline_mode
    for line in lines:
        if mode == singleline_mode:
            pos = line.find(':')
            if pos != -1:
                #indicator was found.
                n = line[:pos].lower()
                v = line[pos+1:]
                d[n] = v.strip()
            elif line == ml_sep:
                #some multiline part start.
                mode = multiline_mode
                mn = ''
                continue
        elif mode == multiline_mode:
            if not mn:
                #First,
                pos = line.find(':')
                if pos != -1:
                    #set name of multiline part
                    mn = line[:pos].lower()
                else:
                    mode == singleline_mode
                if mn in subcontents:
                    mode = subcontent_mode
                    if not d.has_key(mn):
                        d[mn] = []
                        sn = mn
                    sd = {'body':''}
                    continue
                d[mn] = ''
                continue
            elif line == ml_sep:
                #start position of next multiline part
                mn = ''
            else:
                d[mn] = d[mn] + line + chr(0x0a)
        elif mode == subcontent_mode:
            pos = line.find(':')
            if pos != -1:
                #indicator was found.
                n = line[:pos].lower()
                v = line[pos+1:]
                sd[n] = v.strip()
            elif line == ml_sep:
                #some multiline part start.
                mode = multiline_mode
                mn = ''
                d[sn].append(sd)
                continue
            else:
                sd['body'] = sd['body'] + line + chr(0x0a)
    return d

def map_category(catname,catd,cbobj):
    if not catd.has_key(catname):
        #make new category
        cid = cbobj.getCategoryFolder().invokeFactory(id=catname, type_name='COREBlogCategory')
        a_id = 0
        for cat in cbobj.getCategoryFolder().objectValues(['COREBlogCategory']):
            try:
                a_id = max(cat.getInternal_id(),a_id)
            except:
                pass
        catobj = cbobj.getCategoryFolder()[cid]
        catobj.setInternal_id(a_id + 1)
        catobj.setTitle(catname)
        catobj.indexObject()
        catid = catobj.getInternal_id()
        catd[catname] = catid
    else:
        catid = catd[catname]
    return catid,catd

#retrive mt export file
mtobj = container['COREBlog_Export']

#retrive COREBlog instance
cbobj = context

#parse MT export  and find entry area
e = []

#ID - name dictiionary for category
catd = {}

for cat in cbobj.getCategoryFolder().objectValues():
    catd[cat.id] = cat.getInternal_id()

print "start!"

for line in str(mtobj.data).split(chr(0x0a)):
    if line == ent_sep:
        #A etnry block parsed!
        d = parse_block(e)
        #Initialize entry block list
        e = []
        #First, add category
        categoryids = []
        if d.has_key('primary category'):
            cid1,catd = map_category(d['primary category'],catd,cbobj)
            categoryids.append(str(cid1))
        if d.has_key('category'):
            cid2,catd = map_category(d['category'],catd,cbobj)
            subcat = [cid2]
            categoryids.append(str(cid2))
        else:
            subcat = []
        #Add entry
        md = 1
        if d['status'].lower() != 'publish':
            md = 0
        for k in ['extended body','excerpt']:
            if d.has_key(k) and d[k] == chr(0x0a):
                d[k] =''
        eid = cbobj.invokeFactory(id=d['id'], type_name='COREBlogEntry')
        ent = cbobj[eid]
        ent.setTitle(d['title'])
        ent.setBody(d['body'])
        ent.setExtend(d['extended body'])
        ent.setContentType('text/html','body')
        ent.setContentType('text/html','extended')
        ent.setDescription(d['excerpt'])
        ent.setEffectiveDate(d['date'])
        ent.setAllow_comment(str(int(d['allow comments'])+1))
        ent.setReceive_trackback(str(int(d['allow pings'])+1))
        ent.setEntry_categories('\n'.join(categoryids))
        ent.indexObject()
        print "-" * 40
        print d['id'],categoryids
        #add comments
        if d.has_key('comment') and len(d['comment']) > 0:
            for com in d['comment']:
                newid = ent.addComment2Entry(title="Re: " + d['title'],author=com['author'],
                                body=com['body'],email=com['email'],
                                    url=com['url'])
                comobj = cbobj.getCommentFolder()[newid]
                comobj.setEffectiveDate(com['date'])

        #add trackback
        if d.has_key('ping') and len(d['ping']) > 0:
            for tb in d['ping']:
                newid = ent.addTrackback2Entry(title=tb['title'],
                                excerpt=tb['body'],url=tb['url'],
                                blog_name=tb['blog name'])
                comobj = cbobj.getCommentFolder()[newid]
                comobj.setEffectiveDate(tb['date'])
    else:
        e.append(line)

print catd

return printed



## Controller Python Script "resetCookie"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Show archive
##

from DateTime import DateTime

REQUEST = context.REQUEST
form = REQUEST.form

# Set current year,moth

year = DateTime().year()
month = DateTime().month()
day = None

go_redirect = False

if len(traverse_subpath) <= 1:
    # Redirect when called without traverse_subpath,or 
    go_redirect = True
else:
    #Check if all subpath are integer
    try:
        year = int(traverse_subpath[0])
        month = int(traverse_subpath[1])
        if len(traverse_subpath) >= 3:
            day = int(traverse_subpath[2])
    except:
        go_redirect = True

    #Check for number validation
    if year <= 0:
        go_redirect = True
    if month <= 0 or month > 12:
        go_redirect = True

if go_redirect:
    REQUEST.RESPONSE.redirect('../')

return context.cbarchive_view(year=year,month=month,day=day)



#$Id: config.py 82 2005-11-24 06:37:33Z ats $

from Products.CMFCore.CMFCorePermissions import AddPortalContent
from Products.COREBlog2.configuration import zconf

ADD_CONTENT_PERMISSION = 'Add COREBlog2'
PROJECTNAME = "COREBlog2"
SKINS_DIR = 'skins'

GLOBALS = globals()

comment_folder_id = 'comments'
catetory_folder_id = 'categories'
stuff_folder_id = 'stuff'
images_folder_id = 'images'

default_category_id = 'default'

#Relationship id
comemnt_rel_id = 'entrycomments'
trackback_rel_id = 'entrytrackbacks'

#Meta types
coreblog2_meta_type = 'COREBlog2'
coreblogentry_meta_type = 'COREBlogEntry'
coreblogcomment_meta_type = 'COREBlogComment'
coreblogtrackback_meta_type = 'COREBlogTrackback'
coreblogcategory_meta_type = 'COREBlogCategory'

far_far_past = '1000/01/01'
far_far_future= '2999/01/01'


#  ZConfig loader for COREBlog2,
#  based on ATContentTypes
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU Gefneral Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""COREBlog2 ZConfig datatypes

"""
__author__  = 'Atsushi Shibata <shibata@webcore.co.jp>'
__docformat__ = 'plaintext'

from Products.CMFCore import permissions as CMFCorePermissions
from AccessControl import Permissions as ZopePermissions
from ZConfig.datatypes import IdentifierConversion
from ZConfig.datatypes import stock_datatypes


class BaseFactory(object):
    """Basic factory
    """
    
    def __init__(self, section):
        self.name = section.getSectionName()
        #self._parsed = False
        self._section = section
        self._names = {}
        self._parse()
        
    #def __call__(self):
    #    if not self._parsed:
    #        self._parse()
    #    return self
        
    def set(self, name, value):
        self._names[name] = 1
        setattr(self, name, value)
        
    def _parse(self):
        raise NotImplementedError
        
class COREBlog2(BaseFactory):
    """data handler for a coreblog2
    """
    
    def _parse(self):
        sec = self._section
        
        self.set('ping_timeout_default',sec.ping_timeout_default)
        
        # Settings for COREBlog2
        self.set('top_entry_count_default',sec.top_entry_count_default)
        self.set('portlet_item_count_default',sec.portlet_item_count_default)
        self.set('batch_size_default',sec.batch_size_default)
        self.set('dont_send_ping_default',sec.dont_send_ping_default)
        
        # left_slot
        ct = sec.left_slots_default
        if ct is not None:
            slots = tuple(ct.metal_path)
            self.set('left_slots_default',slots)
        ct = sec.right_slots_default
        if ct is not None:
            slots = tuple(ct.metal_path)
            self.set('right_slots_default',slots)
        
        # Settings for Entry
        self.set('description_length',sec.description_length)
        ct = sec.contenttypes
        if ct is not None:
            allowed = tuple(ct.allowed_content_types)
            self.set('allowed_content_types', allowed)



## Return referenced object by kind
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=kind='media'
##title=Show archive
##

# !!!
# Handling for Content-type, File:swf,mp3,mpeg4 or something
#
from Products.CMFCore.utils import getToolByName

INTERFACE = 'Products.COREBlog2.interfaces.IInlineObject.IInlineObject'

itool = getToolByName(context, 'portal_interface')

# Portal types for media
media_types = ('Image','ATVideo',
               'File:video/quicktime','File:video/x-ms-wmv')

ret_l = []

# Get referenced objects
for obj in context.getRefs('relatesTo'):
    try:
        ct = ''
        try:
            ct = ':' + obj.getContentType()
        except:
            pass
        if obj.portal_type in media_types or \
           obj.portal_type + ct in media_types or \
           itool.objectImplements(obj, INTERFACE):
            if kind == 'media':
                ret_l.append(obj)
        elif kind != 'media':
            ret_l.append(obj)
    except:
        pass

return ret_l


##############################################################################
#
# COREBlogCategory.py
# Class for COREBlog2 Category
#
# Copyright (c) 2005 Atsushi Shibata(shibata@webcore.co.jp).
#                                       All Rights Reserved.
#
# Additional imprementation to change superclass to ATCTFolder
#  by Ben <ben [at] smoothify.com> (thanks a lot!)
#
# Permission to use, copy, modify, and distribute this software and its 
# documentation for any purpose and without fee is hereby granted, provided that
# the above copyright notice appear in all copies and that both that copyright 
# notice and this permission notice appear in supporting documentation, and that
# the name of Atsushi Shibata not be used in advertising or publicity pertaining 
# to distribution of the software without specific, written prior permission. 
# 
# ATSUSHI SHIBAT DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
# EVENT SHALL SHIBAT ATSUSHI BE LIABLE FOR ANY SPECIAL, INDIRECT OR 
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
# USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE. 
#
#$Id: coreblogcategory.py 193 2006-04-04 14:11:32Z ats $
#
##############################################################################

#Base classes
from Products.ATContentTypes.content.base import ATCTContent,ATCTFolder,updateAliases
from Products.ATContentTypes.content.schemata import ATContentTypeSchema,\
                                                     finalizeATCTSchema
from Products.ATContentTypes.interfaces import IATBTreeFolder
from Products.Archetypes.public import Schema,registerType
from Products.ATContentTypes.lib.historyaware import HistoryAwareMixin
#Fields
from Products.Archetypes.public import StringField,ImageField,IntegerField
#Widgets
from Products.Archetypes.public import StringWidget,TextAreaWidget,\
                                       ImageWidget,IntegerWidget

from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.COREBlog2.config import PROJECTNAME

#Interfaces
from Products.CMFPlone.interfaces.NonStructuralFolder import \
                                                    INonStructuralFolder

from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import View,ModifyPortalContent
from Products.CMFCore.utils import getToolByName


COREBlogCategorySchema = ATContentTypeSchema.copy() +  Schema((

    IntegerField('internal_id',
        searchable=False,
        isMetadata=True,
        mutator="setInternal_id",
        widget=IntegerWidget(label='Internal ID',
            description='',
            visible={'view':'invisible','edit':'hidden'},
            ),
        ),

    ImageField('category_image',
        widget=ImageWidget(label='Category image',
            description='',
            label_msgid='label_category_image',
            description_msgid='help_category_image',
            i18n_domain='plone',
            ),
        sizes={ 'icon':(16,16)},
        ),

    ),
    marshall=PrimaryFieldMarshaller(),
    )

finalizeATCTSchema(COREBlogCategorySchema)

class COREBlogCategory(ATCTFolder):
    """
    Category class for coreblog2
    """
    
    schema = COREBlogCategorySchema
    archetype_name = "COREBlog Category"
    meta_type = 'COREBlogCategory'
    typeDescription= 'A Category for COREBlog2.'
    typeDescMsgId  = 'coreblog2_description_coreblog2category'
    content_icon = 'coreblogcategory_icon.gif'
    filter_content_types  = 1
    allowed_content_types = []

    __implements__ = ATCTFolder.__implements__ + (INonStructuralFolder,)

    # Not to be shown at add item menu
    global_allow = False
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    
    _at_rename_after_creation = True
    
    # Set up views
    default_view = 'cbcategory_view'
    immediate_view = 'cbcategory_view'
    
    aliases = updateAliases(ATCTFolder,
        {
        'view' : 'cbcategory_view',
        'edit' : 'cbcategory_edit'
        })

    # Not to be shown at navigation
    exclude_from_nav = True

    def canSetDefaultPage(self):
        return False

    # Override initializeArchetype to turn on syndication by default
    def initializeArchetype(self, **kwargs):
       ret_val = ATCTFolder.initializeArchetype(self, **kwargs)
       # Enable topic syndication by default
       syn_tool = getToolByName(self, 'portal_syndication', None)
       if syn_tool is not None:
      
          if (syn_tool.isSiteSyndicationAllowed() and not syn_tool.isSyndicationAllowed(self)):
            syn_tool.enableSyndication(self)
       return ret_val

    security.declareProtected(ModifyPortalContent, 'setInternal_id')
    def setInternal_id(self, value):
        if not self.getInternal_id():
            self.Schema()['internal_id'].set(self,value)

    security.declarePrivate('synContentValues')
    def synContentValues(self):
        """Getter for syndacation support
        """
        syn_tool = getToolByName(self, 'portal_syndication')
        limit = int(syn_tool.getMaxItems(self))

        brains =  self.getEntryInCategory(\
                    category_ids = [self.getInternal_id()],
                    batch=True,
                    sort_order='reverse',
                    b_size=limit,b_start= 0,full_objects=False)

        objs = [brain.getObject() for brain in brains]
        return [obj for obj in objs if obj is not None]

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        # Do internal ID management
        # Find out max category ID

        if not self.getInternal_id():
            a_id = 0
            for cat in container.objectValues([self.meta_type]):
                try:
                    a_id = max(cat.getInternal_id(),a_id)
                except:
                    pass
            self.setInternal_id(a_id + 1)

        #Call base class initialize method
        ATCTFolder.manage_afterAdd(self, item, container)


registerType(COREBlogCategory, PROJECTNAME)



## Return referenced media content presentation
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Return referenced attachment content presentation
##

from Products.CMFCore.utils import getToolByName

# Get plone_utils instance
putil = getToolByName(container,'plone_utils')

# Portal types to list

typesToShow = putil.getUserFriendlyTypes()

for tp in ['COREBlog2','COREBlogEntry','COREBlogCategory',
           'COREBlogComment','COREBlogTrackback',
           'COREBlogCategoryFolder','COREBlogCommentFolder',
           'Folder',]:
    if tp in typesToShow:
        typesToShow.remove(tp)

res = container.portal_catalog.searchResults(sort_on='modified',
                       path='/'.join(context.aq_parent.getPhysicalPath()),
                       portal_type=typesToShow,
                       sort_order='reverse')[:8]

return res



from Interface import Interface, Attribute

class IEnclosedObject(Interface):
    """Marker interface for IEnclosedObject
    """

    def isEnclosedObject():
        """
        Return true if object is "enclosed object".
        """

    def getEnclosureURL():
        """
        Return URL for RSS enclosure tag.
        When you define this method,
        Be sure to imprement getEncosureContentType,getEnclosureSize
        """

    def getEncosureContentType():
        """
        Return ContentType for RSS enclosure tag.
        """

    def getEncosureSize():
        """
        Return size for RSS enclosure tag.
        """

